class bcolors:
    BOLD: str
    BLUE: str
    CYAN: str
    GREEN: str
    MAGENTA: str
    YELLOW: str
    RED: str
    NORMAL: str

def bold(s: str) -> str: ...
def blue(s: str) -> str: ...
def cyan(s: str) -> str: ...
def green(s: str) -> str: ...
def magenta(s: str) -> str: ...
def red(s: str) -> str: ...
def yellow(s: str) -> str: ...
def print_text(s: str) -> None: ...
def print_subtext(s: str) -> None: ...
def print_info(s: str) -> None: ...
def print_done(s: str) -> None: ...
def print_warning(s: str) -> None: ...
def fail(s: str) -> str: ...
